@@ -1,4 +1,4 @@
-/* $Id: tif_win32.c,v 1.38 2011-02-09 18:05:00 fwarmerdam Exp $ */
+/* $Id: tif_win32.c,v 1.39 2011-12-22 17:07:57 bfriesen Exp $ */
 
 /*
  * Copyright (c) 1988-1997 Sam Leffler
@@ -377,7 +377,7 @@ Win32WarningHandler(const char* module, const char* fmt, va_list ap)
 	LPCTSTR szTitleText = "%s Warning";
 	LPCTSTR szDefaultModule = "LIBTIFF";
 	LPCTSTR szTmpModule = (module == NULL) ? szDefaultModule : module;
-        int nBufSize = (strlen(szTmpModule) +
+        SIZE_T nBufSize = (strlen(szTmpModule) +
                         strlen(szTitleText) + strlen(fmt) + 256)*sizeof(char);
 
 	if ((szTitle = (LPTSTR)LocalAlloc(LMEM_FIXED, nBufSize)) == NULL)
@@ -408,7 +408,7 @@ Win32ErrorHandler(const char* module, const char* fmt, va_list ap)
 	LPCTSTR szTitleText = "%s Error";
 	LPCTSTR szDefaultModule = "LIBTIFF";
 	LPCTSTR szTmpModule = (module == NULL) ? szDefaultModule : module;
-        int nBufSize = (strlen(szTmpModule) +
+        SIZE_T nBufSize = (strlen(szTmpModule) +
                         strlen(szTitleText) + strlen(fmt) + 256)*sizeof(char);
 
 	if ((szTitle = (LPTSTR)LocalAlloc(LMEM_FIXED, nBufSize)) == NULL)
